@@ -242,6 +242,41 @@ def test_scatter3d_color():
                color='b', marker='s')
 
 
+def test_scatter3d_depthshade_false():
+    """
+    Test that 3d scatter plot doesn't throw
+    IndexError with depthshade=False (issue #18037)
+    """
+    x = y = z = np.arange(16)
+    fig_test = plt.figure()
+    ax_test = fig_test.add_subplot(projection='3d')
+    ax_test.scatter(x, y, z, depthshade=False)
+
+
+@check_figures_equal(extensions=['png'])
+def test_scatter3d_size(fig_ref, fig_test):
+    """Test that large markers in correct position (issue #18135)"""
+    x = np.arange(10)
+    x, y = np.meshgrid(x, x)
+    z = np.arange(100).reshape(10, 10)
+
+    s = np.full(z.shape, 5)
+    s[0, 0] = 100
+    s[-1, 0] = 100
+    s[0, -1] = 100
+    s[-1, -1] = 100
+
+    ax_ref = fig_ref.gca(projection='3d')
+    ax_test = fig_test.gca(projection='3d')
+
+    small = np.ma.masked_array(z, s == 100, dtype=float)
+    large = np.ma.masked_array(z, s != 100, dtype=float)
+
+    ax_ref.scatter(x, y, large, s=100, c="C0", alpha=1)
+    ax_ref.scatter(x, y, small, s=5, c="C0", alpha=1)
+    ax_test.scatter(x, y, z, s=s, c="C0", alpha=1)
+
+
 @pytest.mark.parametrize('azim', [-50, 130])  # yellow first, blue first
 @check_figures_equal(extensions=['png'])
 def test_marker_draw_order_data_reversed(fig_test, fig_ref, azim):
@@ -1008,3 +1043,19 @@ def test_equal_box_aspect():
     ax.set_zlim3d(XYZlim)
     ax.axis('off')
     ax.set_box_aspect((1, 1, 1))
+
+
+def test_colorbar_pos():
+    num_plots = 2
+    fig, axs = plt.subplots(1, num_plots, figsize=(4, 5),
+                            constrained_layout=True,
+                            subplot_kw={'projection': '3d'})
+    for ax in axs:
+        p_tri = ax.plot_trisurf(np.random.randn(5), np.random.randn(5),
+                                np.random.randn(5))
+
+    cbar = plt.colorbar(p_tri, ax=axs, orientation='horizontal')
+
+    fig.canvas.draw()
+    # check that actually on the bottom
+    assert cbar.ax.get_position().extents[1] < 0.2
